﻿using System;
using System.Threading.Tasks;
using MyCompany.MyProject.Plugin.Cart.Components.CartLine;
using Sitecore.Commerce.Core;
using Sitecore.Commerce.Core.Commands;
using Sitecore.Commerce.Plugin.Carts;

namespace MyCompany.MyProject.Plugin.Cart.Commands
{
    public class UpdatedPersonalisedCartLineCommand : CommerceCommand
    {
        private readonly IUpdateCartLinePipeline _pipeline;
        private readonly IFindEntityPipeline _getPipeline;
        private readonly IServiceProvider _serviceProvider;

        public UpdatedPersonalisedCartLineCommand(IFindEntityPipeline getCartPipeline, IUpdateCartLinePipeline pipeline, IServiceProvider serviceProvider)
          : base(serviceProvider)
        {
            this._getPipeline = getCartPipeline;
            this._pipeline = pipeline;
            _serviceProvider = serviceProvider;
        }

        public virtual async Task<Sitecore.Commerce.Plugin.Carts.Cart> Process(CommerceContext commerceContext, string cartId, string cartLineId, Decimal quantity, string message)
        {
            var cartLineComponent = new CartLineComponent
            {
                Id = cartLineId,
                Quantity = quantity
            };
            
            var personalisationComponent = cartLineComponent.GetComponent<LineItemPersonalisationComponent>();
            personalisationComponent.PersonalMessage = message;

            var updateCartLineCommand =
                new UpdateCartLineCommand(_getPipeline, _pipeline, _serviceProvider);

            return await updateCartLineCommand.Process(commerceContext, cartId, cartLineComponent);
        }
    }
}
