﻿using System;
using System.Threading.Tasks;
using Microsoft.AspNetCore.OData.Builder;
using Sitecore.Commerce.Core;
using Sitecore.Commerce.Core.Commands;
using Sitecore.Framework.Conditions;
using Sitecore.Framework.Pipelines;

namespace MyCompany.MyProject.Plugin.Cart
{
    [PipelineDisplayName("MyCompany.MyProject.Plugin.Cart.ConfigureServiceApiBlock")]
    public class ConfigureServiceApiBlock : PipelineBlock<ODataConventionModelBuilder, ODataConventionModelBuilder, CommercePipelineExecutionContext>
    {
        public override Task<ODataConventionModelBuilder> Run(ODataConventionModelBuilder modelBuilder, CommercePipelineExecutionContext context)
        {
            Condition.Requires(modelBuilder).IsNotNull("The argument can not be null");

            modelBuilder.AddEntityType(typeof(Sitecore.Commerce.Plugin.Carts.Cart));
            modelBuilder.EntitySet<Sitecore.Commerce.Plugin.Carts.Cart>("Carts");
            
            var addActionConfig = modelBuilder.Action(Constants.RouteNames.AddPersonalisedCartLine);
            addActionConfig.Parameter<string>(Constants.RouteParameters.CartId);
            addActionConfig.Parameter<string>(Constants.RouteParameters.ItemId);
            addActionConfig.Parameter<Decimal>(Constants.RouteParameters.Quantity);
            addActionConfig.Parameter<string>(Constants.RouteParameters.PersonalMessage);
            addActionConfig.ReturnsFromEntitySet<CommerceCommand>("Commands");

            var updateActionConfig = modelBuilder.Action(Constants.RouteNames.UpdatePersonalisedCartLine);
            updateActionConfig.Parameter<string>(Constants.RouteParameters.CartId);
            updateActionConfig.Parameter<string>(Constants.RouteParameters.CartLineId);
            updateActionConfig.Parameter<Decimal>(Constants.RouteParameters.Quantity);
            addActionConfig.Parameter<string>(Constants.RouteParameters.PersonalMessage);
            updateActionConfig.ReturnsFromEntitySet<CommerceCommand>("Commands");

            return Task.FromResult(modelBuilder);
        }
    }
}
