﻿using System;
using System.Linq;
using System.Threading.Tasks;
using System.Web.Http.OData;
using Microsoft.AspNetCore.Mvc;
using MyCompany.MyProject.Plugin.Cart.Commands;
using MyCompany.MyProject.Plugin.Cart.Components.CartLine;
using Sitecore.Commerce.Core;
using Sitecore.Commerce.Plugin.Carts;

namespace MyCompany.MyProject.Plugin.Cart.Controllers
{
    public class CommandsController : CommerceController
    {
        public CommandsController(IServiceProvider serviceProvider, CommerceEnvironment globalEnvironment) : base(serviceProvider, globalEnvironment)
        {
        }

        [HttpPut]
        [Route(Constants.RouteNames.AddPersonalisedCartLine + "()")]
        public async Task<IActionResult> AddPersonalisedCartLine([FromBody] ODataActionParameters value)
        {
            //Note the Route name above MUST match the actual name of the method.
            if (!ModelState.IsValid || value == null)
                return new BadRequestObjectResult(ModelState);

            if (!DataHasRequiredKeys(value, 
                Constants.RouteParameters.CartId, 
                Constants.RouteParameters.ItemId, 
                Constants.RouteParameters.Quantity))
            {
                return new BadRequestObjectResult(value);
            }
            
            if (!Decimal.TryParse(value[Constants.RouteParameters.Quantity].ToString(), out var quantity))
            {
                return new BadRequestObjectResult(value);
            }

            var cartId = value[Constants.RouteParameters.CartId].ToString();
            var itemId = value[Constants.RouteParameters.ItemId].ToString();

            CartLineComponent line = new CartLineComponent()
            {
                ItemId = itemId,
                Quantity = quantity
            };

            var message = value[Constants.RouteParameters.PersonalMessage]?.ToString();

            //Add the custom data to the line item.
            line.ChildComponents.Add(new LineItemPersonalisationComponent() { PersonalMessage = message });

            var command = Command<AddCartLineCommand>();

            var cart = await command.Process(CurrentContext, cartId, line).ConfigureAwait(false);
            return new ObjectResult(command);
        }

        [HttpPatch]
        [Route(Constants.RouteNames.UpdatePersonalisedCartLine + "()")]
        public async Task<IActionResult> UpdatePersonalisedCartLine([FromBody] ODataActionParameters parameters)
        {
            if (!ModelState.IsValid || parameters == null)
            {
                return new BadRequestObjectResult(ModelState);
            }

            if (!DataHasRequiredKeys(parameters, Constants.RouteParameters.CartId, Constants.RouteParameters.CartLineId, Constants.RouteParameters.Quantity))
            {
                return new BadRequestObjectResult(parameters);
            }

            var cartId = parameters[Constants.RouteParameters.CartId].ToString();
            var cartLineId = parameters[Constants.RouteParameters.CartLineId].ToString();

            if (!Decimal.TryParse(parameters[Constants.RouteParameters.Quantity].ToString(), out var quantity))
            {
                return new BadRequestObjectResult(parameters);
            }

            var personalMessage = parameters[Constants.RouteParameters.PersonalMessage]?.ToString();

            var command = Command<UpdatedPersonalisedCartLineCommand>();
            var cart = await command.Process(CurrentContext, cartId, cartLineId, quantity, personalMessage).ConfigureAwait(false);
            return new ObjectResult(command);
        }

        private bool DataHasRequiredKeys(ODataActionParameters parameters, params string[] fieldNames)
        {
            return fieldNames.All(field => parameters.ContainsKey(field) && !string.IsNullOrEmpty(parameters[field]?.ToString()));
        }
    }
}
